#define VERSION "0.1"
// Parse the command line.
//
// Call parse_cmdline to parse the command line
// Call parse_print_cmdline to print drive parameter information in command
//   line format
//
// Copyright 2014 David Gesswein.
// This file is part of MFM disk utilities.
//
// MFM disk utilities is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// MFM disk utilities is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with MFM disk utilities.  If not, see <http://www.gnu.org/licenses/>.
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <getopt.h>

#include "msg.h"

#include "parse_cmdline.h"

// Main routine for parsing command lines
//
// argc, argv: Main argc, argv
// drive_parameters: Drive parameters where most of the parsed values are stored
void parse_cmdline(int argc, char *argv[], DRIVE_PARAMS *drive_params)
{
#define MIN_OPTS (0x03)
#define MIN_INITIALIZE_OPTS (0x0f)
   // If you change this fix parse_print_cmdline
   struct option long_options[] = {
         {"file", 1, NULL, 'f'},
         {"drive", 1, NULL, 'd'},
         {"heads", 1, NULL, 'h'},
         {"cylinders", 1, NULL, 'c'},
         {"initialize", 0, NULL, 'i'},
         {"quiet", 1, NULL, 'q'},
         {"version", 0, NULL, 'v'},
         {NULL, 0, NULL, 0}
   };
   char short_options[] = "f:d:ih:c:q:v";
   int rc;
   // Loop counters
   int i;
   int options_index;
   // Bit vector of which options were specified
   uint32_t opt_mask = 0;

   // Enable all errors other than debug
   msg_set_err_mask(~0 ^ MSG_DEBUG);

   // Set defaults
   drive_params->initialize = 0;
   // Handle the options. The long options are converted to the short
   // option name for the switch by getopt_long.
   while ((rc = getopt_long(argc, argv, short_options, long_options,
         &options_index)) != -1) {
      opt_mask |= 1 << options_index;
      switch(rc) {
      case 'h':
         drive_params->num_head = atoi(optarg);
         if (drive_params->num_head > MAX_HEAD) {
            msg(MSG_FATAL,"Maximum number of heads %d:\n", MAX_HEAD);
         }
         break;
      case 'c':
         drive_params->num_cyl = atoi(optarg);
         break;
      case 'f':
         drive_params->filename = optarg;
         break;
      case 'd':
         drive_params->drive = atoi(optarg);
         break;
      case 'i':
         drive_params->initialize = 1;
         break;
      case 'q':
         msg_set_err_mask(~strtol(optarg, NULL, 0));
         break;
      case 'v':
         msg(MSG_INFO_SUMMARY,"Version %s\n",VERSION);
         break;
      case '?':
         msg(MSG_FATAL,"Valid options:\n");
         for (i = 0; long_options[i].name != NULL; i++) {
            msg(MSG_FATAL, "%c %s\n", long_options[i].val, long_options[i].name);
         }
         exit(1);
         break;
      default:
         msg(MSG_FATAL, "Didn't process argument %c\n", rc);
         exit(1);
      }
   }
   if ((opt_mask & MIN_OPTS) != MIN_OPTS) {
      msg(MSG_FATAL, "Program requires options:");
      for (i = 0; i < 32; i++) {
         int bit = (1 << i);
         if (!(opt_mask & bit) && (MIN_OPTS & bit)) {
            msg(MSG_FATAL, " %s", long_options[i].name);
         }
      }
      msg(MSG_FATAL, "\n");
      exit(1);
   }
   if (drive_params->initialize && (opt_mask & MIN_INITIALIZE_OPTS) != 
         MIN_INITIALIZE_OPTS) {
      msg(MSG_FATAL, "Program requires options:");
      for (i = 0; i < 32; i++) {
         int bit = (1 << i);
         if (!(opt_mask & bit) && (MIN_INITIALIZE_OPTS & bit)) {
            msg(MSG_FATAL, " %s", long_options[i].name);
         }
      }
      msg(MSG_FATAL, "\n");
      exit(1);
   }
}
